@@ -8,21 +8,27 @@
 	unstable_name_collisions,
 	unused_assignments
 )]
-#![deny(unstable_name_collisions)]
 #![deny(clippy::all, clippy::perf, clippy::nursery, clippy::pedantic)]
-#![deny(clippy::filetype_is_file)]
-#![deny(clippy::cargo)]
-#![deny(clippy::unwrap_used)]
-#![deny(clippy::panic)]
-#![deny(clippy::match_like_matches_macro)]
-#![deny(clippy::needless_update)]
-#![allow(clippy::module_name_repetitions)]
-#![allow(clippy::must_use_candidate)]
-#![allow(clippy::missing_errors_doc)]
-//TODO: get this in someday since expect still leads us to crashes sometimes
-// #![deny(clippy::expect_used)]
-//TODO: consider cleaning some up and allow specific places
-#![allow(clippy::significant_drop_tightening)]
+#![deny(
+	clippy::filetype_is_file,
+	clippy::cargo,
+	clippy::unwrap_used,
+	clippy::panic,
+	clippy::match_like_matches_macro,
+	clippy::needless_update
+	//TODO: get this in someday since expect still leads us to crashes sometimes
+	// clippy::expect_used
+)]
+#![allow(
+	clippy::module_name_repetitions,
+	clippy::must_use_candidate,
+	clippy::missing_errors_doc
+)]
+//TODO:
+#![allow(
+	clippy::significant_drop_tightening,
+	clippy::missing_panics_doc
+)]
 
 pub mod asyncjob;
 mod blame;
@@ -32,6 +38,7 @@ mod commit_files;
 mod diff;
 mod error;
 mod fetch_job;
+mod filter_commits;
 mod progress;
 mod pull;
 mod push;
@@ -51,6 +58,7 @@ pub use crate::{
 	diff::{AsyncDiff, DiffParams, DiffType},
 	error::{Error, Result},
 	fetch_job::AsyncFetchJob,
+	filter_commits::{AsyncCommitFilterJob, CommitFilterResult},
 	progress::ProgressPercent,
 	pull::{AsyncPull, FetchRequest},
 	push::{AsyncPush, PushRequest},
@@ -105,6 +113,8 @@ pub enum AsyncGitNotification {
 	Branches,
 	///
 	TreeFiles,
+	///
+	CommitFilter,
 }
 
 /// helper function to calculate the hash of an arbitrary type that implements the `Hash` trait
