@@ -1,7 +1,9 @@
 use crate::{
 	error::Result,
-	sync::{repo, CommitId, LogWalker, LogWalkerFilter, RepoPath},
-	AsyncGitNotification,
+	sync::{
+		repo, CommitId, LogWalker, RepoPath, SharedCommitFilterFn,
+	},
+	AsyncGitNotification, Error,
 };
 use crossbeam_channel::Sender;
 use scopetime::scope_time;
@@ -11,11 +13,11 @@ use std::{
 		Arc, Mutex,
 	},
 	thread,
-	time::Duration,
+	time::{Duration, Instant},
 };
 
 ///
-#[derive(PartialEq, Eq)]
+#[derive(PartialEq, Eq, Debug)]
 pub enum FetchStatus {
 	/// previous fetch still running
 	Pending,
@@ -25,14 +27,22 @@ pub enum FetchStatus {
 	Started,
 }
 
+///
+pub struct AsyncLogResult {
+	///
+	pub commits: Vec<CommitId>,
+	///
+	pub duration: Duration,
+}
 ///
 pub struct AsyncLog {
-	current: Arc<Mutex<Vec<CommitId>>>,
+	current: Arc<Mutex<AsyncLogResult>>,
 	current_head: Arc<Mutex<Option<CommitId>>>,
 	sender: Sender<AsyncGitNotification>,
 	pending: Arc<AtomicBool>,
 	background: Arc<AtomicBool>,
-	filter: Option<LogWalkerFilter>,
+	filter: Option<SharedCommitFilterFn>,
+	partial_extract: AtomicBool,
 	repo: RepoPath,
 }
 
@@ -45,22 +55,26 @@ impl AsyncLog {
 	pub fn new(
 		repo: RepoPath,
 		sender: &Sender<AsyncGitNotification>,
-		filter: Option<LogWalkerFilter>,
+		filter: Option<SharedCommitFilterFn>,
 	) -> Self {
 		Self {
 			repo,
-			current: Arc::new(Mutex::new(Vec::new())),
+			current: Arc::new(Mutex::new(AsyncLogResult {
+				commits: Vec::new(),
+				duration: Duration::default(),
+			})),
 			current_head: Arc::new(Mutex::new(None)),
 			sender: sender.clone(),
 			pending: Arc::new(AtomicBool::new(false)),
 			background: Arc::new(AtomicBool::new(false)),
 			filter,
+			partial_extract: AtomicBool::new(false),
 		}
 	}
 
 	///
 	pub fn count(&self) -> Result<usize> {
-		Ok(self.current.lock()?.len())
+		Ok(self.current.lock()?.commits.len())
 	}
 
 	///
@@ -69,7 +83,11 @@ impl AsyncLog {
 		start_index: usize,
 		amount: usize,
 	) -> Result<Vec<CommitId>> {
-		let list = self.current.lock()?;
+		if self.partial_extract.load(Ordering::Relaxed) {
+			return Err(Error::Generic(String::from("Faulty usage of AsyncLog: Cannot partially extract items and rely on get_items slice to still work!")));
+		}
+
+		let list = &self.current.lock()?.commits;
 		let list_len = list.len();
 		let min = start_index.min(list_len);
 		let max = min + amount;
@@ -78,11 +96,27 @@ impl AsyncLog {
 	}
 
 	///
-	pub fn position(&self, id: CommitId) -> Result<Option<usize>> {
-		let list = self.current.lock()?;
-		let position = list.iter().position(|&x| x == id);
+	pub fn get_items(&self) -> Result<Vec<CommitId>> {
+		if self.partial_extract.load(Ordering::Relaxed) {
+			return Err(Error::Generic(String::from("Faulty usage of AsyncLog: Cannot partially extract items and rely on get_items slice to still work!")));
+		}
 
-		Ok(position)
+		let list = &self.current.lock()?.commits;
+		Ok(list.clone())
+	}
+
+	///
+	pub fn extract_items(&self) -> Result<Vec<CommitId>> {
+		self.partial_extract.store(true, Ordering::Relaxed);
+		let list = &mut self.current.lock()?.commits;
+		let result = list.clone();
+		list.clear();
+		Ok(result)
+	}
+
+	///
+	pub fn get_last_duration(&self) -> Result<Duration> {
+		Ok(self.current.lock()?.duration)
 	}
 
 	///
@@ -122,6 +156,8 @@ impl AsyncLog {
 			return Ok(FetchStatus::NoChange);
 		}
 
+		self.pending.store(true, Ordering::Relaxed);
+
 		self.clear()?;
 
 		let arc_current = Arc::clone(&self.current);
@@ -131,8 +167,6 @@ impl AsyncLog {
 		let filter = self.filter.clone();
 		let repo_path = self.repo.clone();
 
-		self.pending.store(true, Ordering::Relaxed);
-
 		if let Ok(head) = repo(&self.repo)?.head() {
 			*self.current_head.lock()? =
 				head.target().map(CommitId::new);
@@ -160,25 +194,27 @@ impl AsyncLog {
 
 	fn fetch_helper(
 		repo_path: &RepoPath,
-		arc_current: &Arc<Mutex<Vec<CommitId>>>,
+		arc_current: &Arc<Mutex<AsyncLogResult>>,
 		arc_background: &Arc<AtomicBool>,
 		sender: &Sender<AsyncGitNotification>,
-		filter: Option<LogWalkerFilter>,
+		filter: Option<SharedCommitFilterFn>,
 	) -> Result<()> {
+		let start_time = Instant::now();
+
 		let mut entries = Vec::with_capacity(LIMIT_COUNT);
 		let r = repo(repo_path)?;
 		let mut walker =
 			LogWalker::new(&r, LIMIT_COUNT)?.filter(filter);
+
 		loop {
 			entries.clear();
-			let res_is_err = walker.read(&mut entries).is_err();
+			let read = walker.read(&mut entries)?;
 
-			if !res_is_err {
-				let mut current = arc_current.lock()?;
-				current.extend(entries.iter());
-			}
+			let mut current = arc_current.lock()?;
+			current.commits.extend(entries.iter());
+			current.duration = start_time.elapsed();
 
-			if res_is_err || entries.len() <= 1 {
+			if read == 0 {
 				break;
 			}
 			Self::notify(sender);
@@ -189,15 +225,19 @@ impl AsyncLog {
 				} else {
 					SLEEP_FOREGROUND
 				};
+
 			thread::sleep(sleep_duration);
 		}
 
+		log::trace!("revlog visited: {}", walker.visited());
+
 		Ok(())
 	}
 
 	fn clear(&mut self) -> Result<()> {
-		self.current.lock()?.clear();
+		self.current.lock()?.commits.clear();
 		*self.current_head.lock()? = None;
+		self.partial_extract.store(false, Ordering::Relaxed);
 		Ok(())
 	}
 
