@@ -53,6 +53,11 @@ impl FileTree {
 		self.items.file_count() == 0
 	}
 
+	///
+	pub const fn selection(&self) -> Option<usize> {
+		self.selection
+	}
+
 	///
 	pub fn collapse_but_root(&mut self) {
 		if !self.is_empty() {
