@@ -2,12 +2,13 @@ use crate::{
 	accessors,
 	cmdbar::CommandBar,
 	components::{
-		event_pump, AppOption, BlameFileComponent, BranchFindPopup,
-		BranchListComponent, CommandBlocking, CommandInfo,
-		CommitComponent, CompareCommitsComponent, Component,
-		ConfirmComponent, CreateBranchComponent, DrawableComponent,
-		ExternalEditorComponent, FetchComponent, FileFindPopup,
-		FileRevlogComponent, HelpComponent, InspectCommitComponent,
+		command_pump, event_pump, AppOption, BlameFileComponent,
+		BranchListComponent, CommandInfo, CommitComponent,
+		CompareCommitsComponent, Component, ConfirmComponent,
+		CreateBranchComponent, DrawableComponent,
+		ExternalEditorComponent, FetchComponent, FileRevlogComponent,
+		FuzzyFindPopup, FuzzyFinderTarget, HelpComponent,
+		InspectCommitComponent, LogSearchPopupComponent,
 		MsgComponent, OptionsPopupComponent, PullComponent,
 		PushComponent, PushTagsComponent, RenameBranchComponent,
 		ResetPopupComponent, RevisionFilesPopup, StashMsgComponent,
@@ -19,7 +20,8 @@ use crate::{
 	options::{Options, SharedOptions},
 	popup_stack::PopupStack,
 	queue::{
-		Action, InternalEvent, NeedsUpdate, Queue, StackablePopupOpen,
+		Action, AppTabs, InternalEvent, NeedsUpdate, Queue,
+		StackablePopupOpen,
 	},
 	setup_popups,
 	strings::{self, ellipsis_trim_start, order},
@@ -45,7 +47,7 @@ use ratatui::{
 };
 use std::{
 	cell::{Cell, RefCell},
-	path::Path,
+	path::{Path, PathBuf},
 	rc::Rc,
 };
 use unicode_width::UnicodeWidthStr;
@@ -72,8 +74,8 @@ pub struct App {
 	compare_commits_popup: CompareCommitsComponent,
 	external_editor_popup: ExternalEditorComponent,
 	revision_files_popup: RevisionFilesPopup,
-	find_file_popup: FileFindPopup,
-	branch_find_popup: BranchFindPopup,
+	fuzzy_find_popup: FuzzyFindPopup,
+	log_search_popup: LogSearchPopupComponent,
 	push_popup: PushComponent,
 	push_tags_popup: PushTagsComponent,
 	pull_popup: PullComponent,
@@ -271,12 +273,13 @@ impl App {
 				theme.clone(),
 				key_config.clone(),
 			),
-			find_file_popup: FileFindPopup::new(
+			log_search_popup: LogSearchPopupComponent::new(
+				repo.clone(),
 				&queue,
 				theme.clone(),
 				key_config.clone(),
 			),
-			branch_find_popup: BranchFindPopup::new(
+			fuzzy_find_popup: FuzzyFindPopup::new(
 				&queue,
 				theme.clone(),
 				key_config.clone(),
@@ -585,8 +588,8 @@ impl App {
 	accessors!(
 		self,
 		[
-			find_file_popup,
-			branch_find_popup,
+			log_search_popup,
+			fuzzy_find_popup,
 			msg,
 			reset,
 			commit,
@@ -637,8 +640,8 @@ impl App {
 			create_branch_popup,
 			rename_branch_popup,
 			revision_files_popup,
-			find_file_popup,
-			branch_find_popup,
+			fuzzy_find_popup,
+			log_search_popup,
 			push_popup,
 			push_tags_popup,
 			pull_popup,
@@ -695,15 +698,15 @@ impl App {
 
 	fn switch_tab(&mut self, k: &KeyEvent) -> Result<()> {
 		if key_match(k, self.key_config.keys.tab_status) {
-			self.set_tab(0)?;
+			self.switch_to_tab(&AppTabs::Status)?;
 		} else if key_match(k, self.key_config.keys.tab_log) {
-			self.set_tab(1)?;
+			self.switch_to_tab(&AppTabs::Log)?;
 		} else if key_match(k, self.key_config.keys.tab_files) {
-			self.set_tab(2)?;
+			self.switch_to_tab(&AppTabs::Files)?;
 		} else if key_match(k, self.key_config.keys.tab_stashing) {
-			self.set_tab(3)?;
+			self.switch_to_tab(&AppTabs::Stashing)?;
 		} else if key_match(k, self.key_config.keys.tab_stashes) {
-			self.set_tab(4)?;
+			self.switch_to_tab(&AppTabs::Stashlist)?;
 		}
 
 		Ok(())
@@ -725,6 +728,17 @@ impl App {
 		Ok(())
 	}
 
+	fn switch_to_tab(&mut self, tab: &AppTabs) -> Result<()> {
+		match tab {
+			AppTabs::Status => self.set_tab(0)?,
+			AppTabs::Log => self.set_tab(1)?,
+			AppTabs::Files => self.set_tab(2)?,
+			AppTabs::Stashing => self.set_tab(3)?,
+			AppTabs::Stashlist => self.set_tab(4)?,
+		}
+		Ok(())
+	}
+
 	fn update_commands(&mut self) {
 		if self.help.is_visible() {
 			self.help.set_cmds(self.commands(true));
@@ -853,6 +867,10 @@ impl App {
 				self.tags_popup.open()?;
 			}
 			InternalEvent::TabSwitchStatus => self.set_tab(0)?,
+			InternalEvent::TabSwitch(tab) => {
+				self.switch_to_tab(&tab)?;
+				flags.insert(NeedsUpdate::ALL);
+			}
 			InternalEvent::SelectCommitInRevlog(id) => {
 				if let Err(error) = self.revlog.select_commit(id) {
 					self.queue.push(InternalEvent::ShowErrorMsg(
@@ -897,13 +915,13 @@ impl App {
 			InternalEvent::StatusLastFileMoved => {
 				self.status_tab.last_file_moved()?;
 			}
-			InternalEvent::OpenFileFinder(files) => {
-				self.find_file_popup.open(&files)?;
+			InternalEvent::OpenFuzzyFinder(contents, target) => {
+				self.fuzzy_find_popup.open(contents, target)?;
 				flags
 					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
 			}
-			InternalEvent::OpenBranchFinder(branches) => {
-				self.branch_find_popup.open(branches)?;
+			InternalEvent::OpenLogSearchPopup => {
+				self.log_search_popup.open()?;
 				flags
 					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
 			}
@@ -921,14 +939,25 @@ impl App {
 
 				flags.insert(NeedsUpdate::ALL);
 			}
-			InternalEvent::FileFinderChanged(file) => {
-				self.files_tab.file_finder_update(&file);
-				self.revision_files_popup.file_finder_update(&file);
-				flags
-					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
-			}
-			InternalEvent::BranchFinderChanged(idx) => {
-				self.select_branch_popup.branch_finder_update(idx)?;
+			InternalEvent::FuzzyFinderChanged(
+				idx,
+				content,
+				target,
+			) => {
+				match target {
+					FuzzyFinderTarget::Branches => self
+						.select_branch_popup
+						.branch_finder_update(idx)?,
+					FuzzyFinderTarget::Files => {
+						self.files_tab.file_finder_update(
+							&PathBuf::from(content.clone()),
+						);
+						self.revision_files_popup.file_finder_update(
+							&PathBuf::from(content),
+						);
+					}
+				}
+
 				flags
 					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
 			}
@@ -962,12 +991,14 @@ impl App {
 			InternalEvent::OpenResetPopup(id) => {
 				self.reset_popup.open(id)?;
 			}
+			InternalEvent::CommitSearch(options) => {
+				self.revlog.search(options);
+			}
 		};
 
 		Ok(flags)
 	}
 
-	#[allow(clippy::too_many_lines)]
 	fn process_confirmed_action(
 		&mut self,
 		action: Action,
@@ -975,9 +1006,7 @@ impl App {
 	) -> Result<()> {
 		match action {
 			Action::Reset(r) => {
-				if self.status_tab.reset(&r) {
-					flags.insert(NeedsUpdate::ALL);
-				}
+				self.status_tab.reset(&r);
 			}
 			Action::StashDrop(_) | Action::StashPop(_) => {
 				if let Err(e) = self
@@ -988,12 +1017,14 @@ impl App {
 						e.to_string(),
 					));
 				}
-
-				flags.insert(NeedsUpdate::ALL);
 			}
 			Action::ResetHunk(path, hash) => {
-				sync::reset_hunk(&self.repo.borrow(), &path, hash)?;
-				flags.insert(NeedsUpdate::ALL);
+				sync::reset_hunk(
+					&self.repo.borrow(),
+					&path,
+					hash,
+					Some(self.options.borrow().diff_options()),
+				)?;
 			}
 			Action::ResetLines(path, lines) => {
 				sync::discard_lines(
@@ -1001,7 +1032,6 @@ impl App {
 					&path,
 					&lines,
 				)?;
-				flags.insert(NeedsUpdate::ALL);
 			}
 			Action::DeleteLocalBranch(branch_ref) => {
 				if let Err(e) = sync::delete_branch(
@@ -1012,50 +1042,14 @@ impl App {
 						e.to_string(),
 					));
 				}
-				flags.insert(NeedsUpdate::ALL);
+
 				self.select_branch_popup.update_branches()?;
 			}
 			Action::DeleteRemoteBranch(branch_ref) => {
-				self.queue.push(
-					//TODO: check if this is correct based on the fix in `c6abbaf`
-					branch_ref.rsplit('/').next().map_or_else(
-						|| {
-							InternalEvent::ShowErrorMsg(format!(
-						"Failed to find the branch name in {branch_ref}"
-					))
-						},
-						|name| {
-							InternalEvent::Push(
-								name.to_string(),
-								PushType::Branch,
-								false,
-								true,
-							)
-						},
-					),
-				);
-				flags.insert(NeedsUpdate::ALL);
-				self.select_branch_popup.update_branches()?;
+				self.delete_remote_branch(&branch_ref)?;
 			}
 			Action::DeleteTag(tag_name) => {
-				if let Err(error) =
-					sync::delete_tag(&self.repo.borrow(), &tag_name)
-				{
-					self.queue.push(InternalEvent::ShowErrorMsg(
-						error.to_string(),
-					));
-				} else {
-					let remote = sync::get_default_remote(
-						&self.repo.borrow(),
-					)?;
-
-					self.queue.push(InternalEvent::ConfirmAction(
-						Action::DeleteRemoteTag(tag_name, remote),
-					));
-
-					flags.insert(NeedsUpdate::ALL);
-					self.tags_popup.update_tags()?;
-				}
+				self.delete_tag(tag_name)?;
 			}
 			Action::DeleteRemoteTag(tag_name, _remote) => {
 				self.queue.push(InternalEvent::Push(
@@ -1075,36 +1069,75 @@ impl App {
 			}
 			Action::PullMerge { rebase, .. } => {
 				self.pull_popup.try_conflict_free_merge(rebase);
-				flags.insert(NeedsUpdate::ALL);
 			}
 			Action::AbortRevert | Action::AbortMerge => {
 				self.status_tab.revert_pending_state();
-				flags.insert(NeedsUpdate::ALL);
 			}
 			Action::AbortRebase => {
 				self.status_tab.abort_rebase();
-				flags.insert(NeedsUpdate::ALL);
 			}
 		};
 
+		flags.insert(NeedsUpdate::ALL);
+
+		Ok(())
+	}
+
+	fn delete_tag(&mut self, tag_name: String) -> Result<()> {
+		if let Err(error) =
+			sync::delete_tag(&self.repo.borrow(), &tag_name)
+		{
+			self.queue
+				.push(InternalEvent::ShowErrorMsg(error.to_string()));
+		} else {
+			let remote =
+				sync::get_default_remote(&self.repo.borrow())?;
+
+			self.queue.push(InternalEvent::ConfirmAction(
+				Action::DeleteRemoteTag(tag_name, remote),
+			));
+
+			self.tags_popup.update_tags()?;
+		};
+		Ok(())
+	}
+
+	fn delete_remote_branch(
+		&mut self,
+		branch_ref: &str,
+	) -> Result<()> {
+		self.queue.push(
+			//TODO: check if this is correct based on the fix in `c6abbaf`
+			branch_ref.rsplit('/').next().map_or_else(
+				|| {
+					InternalEvent::ShowErrorMsg(format!(
+						    "Failed to find the branch name in {branch_ref}"
+					    ))
+				},
+				|name| {
+					InternalEvent::Push(
+						name.to_string(),
+						PushType::Branch,
+						false,
+						true,
+					)
+				},
+			),
+		);
+
+		self.select_branch_popup.update_branches()?;
+
 		Ok(())
 	}
 
 	fn commands(&self, force_all: bool) -> Vec<CommandInfo> {
 		let mut res = Vec::new();
 
-		for c in self.components() {
-			if c.commands(&mut res, force_all)
-				!= CommandBlocking::PassingOn
-				&& !force_all
-			{
-				break;
-			}
-		}
+		command_pump(&mut res, force_all, &self.components());
 
 		res.push(CommandInfo::new(
 			strings::commands::find_file(&self.key_config),
-			!self.find_file_popup.is_visible(),
+			!self.fuzzy_find_popup.is_visible(),
 			(!self.any_popup_visible()
 				&& self.files_tab.is_visible())
 				|| self.revision_files_popup.is_visible()
