@@ -1,5 +1,4 @@
 mod blame_file;
-mod branch_find_popup;
 mod branchlist;
 mod changes;
 mod command;
@@ -12,10 +11,11 @@ mod cred;
 mod diff;
 mod externaleditor;
 mod fetch;
-mod file_find_popup;
 mod file_revlog;
+mod fuzzy_find_popup;
 mod help;
 mod inspect_commit;
+mod log_search_popup;
 mod msg;
 mod options_popup;
 mod pull;
@@ -37,7 +37,6 @@ mod utils;
 
 pub use self::status_tree::StatusTreeComponent;
 pub use blame_file::{BlameFileComponent, BlameFileOpen};
-pub use branch_find_popup::BranchFindPopup;
 pub use branchlist::BranchListComponent;
 pub use changes::ChangesComponent;
 pub use command::{CommandInfo, CommandText};
@@ -49,10 +48,11 @@ pub use create_branch::CreateBranchComponent;
 pub use diff::DiffComponent;
 pub use externaleditor::ExternalEditorComponent;
 pub use fetch::FetchComponent;
-pub use file_find_popup::FileFindPopup;
 pub use file_revlog::{FileRevOpen, FileRevlogComponent};
+pub use fuzzy_find_popup::FuzzyFindPopup;
 pub use help::HelpComponent;
 pub use inspect_commit::{InspectCommitComponent, InspectCommitOpen};
+pub use log_search_popup::LogSearchPopupComponent;
 pub use msg::MsgComponent;
 pub use options_popup::{AppOption, OptionsPopupComponent};
 pub use pull::PullComponent;
@@ -236,6 +236,12 @@ pub enum EventState {
 	NotConsumed,
 }
 
+#[derive(Copy, Clone)]
+pub enum FuzzyFinderTarget {
+	Branches,
+	Files,
+}
+
 impl EventState {
 	pub fn is_consumed(&self) -> bool {
 		*self == Self::Consumed
