@@ -1,6 +1,7 @@
 use asyncgit::sync::{CommitId, CommitInfo};
 use chrono::{DateTime, Duration, Local, NaiveDateTime, Utc};
-use std::slice::Iter;
+use indexmap::IndexSet;
+use std::{rc::Rc, slice::Iter};
 
 #[cfg(feature = "ghemoji")]
 use super::emoji::emojifi_string;
@@ -18,6 +19,7 @@ pub struct LogEntry {
 	//TODO: use tinyvec here
 	pub hash_short: BoxStr,
 	pub id: CommitId,
+	pub highlighted: bool,
 }
 
 impl From<CommitInfo> for LogEntry {
@@ -49,6 +51,7 @@ impl From<CommitInfo> for LogEntry {
 			time,
 			hash_short,
 			id: c.id,
+			highlighted: false,
 		}
 	}
 }
@@ -74,20 +77,31 @@ impl LogEntry {
 ///
 #[derive(Default)]
 pub struct ItemBatch {
-	index_offset: usize,
+	index_offset: Option<usize>,
 	items: Vec<LogEntry>,
+	highlighting: bool,
 }
 
 impl ItemBatch {
 	fn last_idx(&self) -> usize {
-		self.index_offset + self.items.len()
+		self.index_offset() + self.items.len()
 	}
 
 	///
-	pub const fn index_offset(&self) -> usize {
+	pub fn index_offset(&self) -> usize {
+		self.index_offset.unwrap_or_default()
+	}
+
+	///
+	pub const fn index_offset_raw(&self) -> Option<usize> {
 		self.index_offset
 	}
 
+	///
+	pub const fn highlighting(&self) -> bool {
+		self.highlighting
+	}
+
 	/// shortcut to get an `Iter` of our internal items
 	pub fn iter(&self) -> Iter<'_, LogEntry> {
 		self.items.iter()
@@ -96,6 +110,7 @@ impl ItemBatch {
 	/// clear curent list of items
 	pub fn clear(&mut self) {
 		self.items.clear();
+		self.index_offset = None;
 	}
 
 	/// insert new batch of items
@@ -103,10 +118,27 @@ impl ItemBatch {
 		&mut self,
 		start_index: usize,
 		commits: Vec<CommitInfo>,
+		highlighted: &Option<Rc<IndexSet<CommitId>>>,
 	) {
-		self.items.clear();
-		self.items.extend(commits.into_iter().map(LogEntry::from));
-		self.index_offset = start_index;
+		self.clear();
+
+		if !commits.is_empty() {
+			self.items.extend(commits.into_iter().map(|c| {
+				let id = c.id;
+				let mut entry = LogEntry::from(c);
+				if highlighted
+					.as_ref()
+					.map(|highlighted| highlighted.contains(&id))
+					.unwrap_or_default()
+				{
+					entry.highlighted = true;
+				}
+				entry
+			}));
+
+			self.index_offset = Some(start_index);
+			self.highlighting = highlighted.is_some();
+		}
 	}
 
 	/// returns `true` if we should fetch updated list of items
@@ -117,7 +149,7 @@ impl ItemBatch {
 			.saturating_add(SLICE_OFFSET_RELOAD_THRESHOLD)
 			.min(idx_max);
 
-		let needs_data_top = want_min < self.index_offset;
+		let needs_data_top = want_min < self.index_offset();
 		let needs_data_bottom = want_max >= self.last_idx();
 		needs_data_bottom || needs_data_top
 	}
