@@ -38,7 +38,6 @@ mod keys;
 mod notify_mutex;
 mod options;
 mod popup_stack;
-mod profiler;
 mod queue;
 mod spinner;
 mod string_utils;
@@ -66,7 +65,6 @@ use crossterm::{
 };
 use input::{Input, InputEvent, InputState};
 use keys::KeyConfig;
-use profiler::Profiler;
 use ratatui::{
 	backend::{Backend, CrosstermBackend},
 	Terminal,
@@ -127,8 +125,6 @@ fn main() -> Result<()> {
 
 	let cliargs = process_cmdline()?;
 
-	let _profiler = Profiler::new();
-
 	asyncgit::register_tracing_logging();
 
 	if !valid_path(&cliargs.repo_path) {
@@ -138,9 +134,7 @@ fn main() -> Result<()> {
 	let key_config = KeyConfig::init()
 		.map_err(|e| eprintln!("KeyConfig loading error: {e}"))
 		.unwrap_or_default();
-	let theme = Theme::init(&cliargs.theme)
-		.map_err(|e| eprintln!("Theme loading error: {e}"))
-		.unwrap_or_default();
+	let theme = Theme::init(&cliargs.theme);
 
 	setup_terminal()?;
 	defer! {
@@ -325,7 +319,11 @@ fn draw<B: Backend>(
 }
 
 fn valid_path(repo_path: &RepoPath) -> bool {
-	asyncgit::sync::is_repo(repo_path)
+	let error = asyncgit::sync::repo_open_error(repo_path);
+	if let Some(error) = &error {
+		eprintln!("repo open error: {error}");
+	}
+	error.is_none()
 }
 
 fn select_event(
@@ -357,7 +355,7 @@ fn select_event(
 			QueueEvent::AsyncEvent(AsyncNotification::App(e))
 		}),
 		3 => oper.recv(rx_ticker).map(|_| QueueEvent::Notify),
-		4 => oper.recv(rx_notify).map(|_| QueueEvent::Notify),
+		4 => oper.recv(rx_notify).map(|()| QueueEvent::Notify),
 		5 => oper.recv(rx_spinner).map(|_| QueueEvent::SpinnerUpdate),
 		_ => bail!("unknown select source"),
 	}?;
