@@ -1,5 +1,5 @@
+use ratatui::{backend::Backend, Terminal};
 use std::{cell::Cell, char, io};
-use tui::{backend::Backend, Terminal};
 
 // static SPINNER_CHARS: &[char] = &['◢', '◣', '◤', '◥'];
 // static SPINNER_CHARS: &[char] = &['⢹', '⢺', '⢼', '⣸', '⣇', '⡧', '⡗', '⡏'];
@@ -48,7 +48,7 @@ impl Spinner {
 		if self.last_char.get() != char_to_draw {
 			self.last_char.set(char_to_draw);
 
-			let c = tui::buffer::Cell::default()
+			let c = ratatui::buffer::Cell::default()
 				.set_char(char_to_draw)
 				.clone();
 
@@ -56,7 +56,7 @@ impl Spinner {
 				.backend_mut()
 				.draw(vec![(0_u16, 0_u16, &c)].into_iter())?;
 
-			tui::backend::Backend::flush(terminal.backend_mut())?;
+			ratatui::backend::Backend::flush(terminal.backend_mut())?;
 		}
 
 		Ok(())
