@@ -48,14 +48,7 @@ impl StashList {
 	pub fn update(&mut self) -> Result<()> {
 		if self.is_visible() {
 			let stashes = sync::get_stashes(&self.repo.borrow())?;
-			let commits = sync::get_commits_info(
-				&self.repo.borrow(),
-				stashes.as_slice(),
-				100,
-			)?;
-
-			self.list.set_count_total(commits.len());
-			self.list.items().set_items(0, commits);
+			self.list.set_commits(stashes);
 		}
 
 		Ok(())
@@ -65,7 +58,7 @@ impl StashList {
 		if let Some(e) = self.list.selected_entry() {
 			match sync::stash_apply(&self.repo.borrow(), e.id, false)
 			{
-				Ok(_) => {
+				Ok(()) => {
 					self.queue.push(InternalEvent::TabSwitchStatus);
 				}
 				Err(e) => {
@@ -143,6 +136,8 @@ impl StashList {
 		self.list.clear_marked();
 		self.update()?;
 
+		self.queue.push(InternalEvent::TabSwitchStatus);
+
 		Ok(())
 	}
 }
