@@ -10,7 +10,7 @@ use crate::{
 	options::SharedOptions,
 	queue::{Action, InternalEvent, NeedsUpdate, Queue, ResetItem},
 	strings, try_or_popup,
-	ui::style::SharedTheme,
+	ui::style::{SharedTheme, Theme},
 };
 use anyhow::Result;
 use asyncgit::{
@@ -195,6 +195,7 @@ impl Status {
 				theme,
 				key_config.clone(),
 				false,
+				options.clone(),
 			),
 			git_diff: AsyncDiff::new(repo_clone.clone(), sender),
 			git_status_workdir: AsyncStatus::new(
@@ -312,9 +313,7 @@ impl Status {
 					Block::default()
 						.border_type(BorderType::Plain)
 						.borders(Borders::all())
-						.border_style(
-							Style::default().fg(Color::Yellow),
-						)
+						.border_style(Theme::attention_block())
 						.title(format!(
 							"Pending {:?}",
 							self.git_state
@@ -449,6 +448,10 @@ impl Status {
 		&mut self,
 		ev: AsyncGitNotification,
 	) -> Result<()> {
+		if !self.is_visible() {
+			return Ok(());
+		}
+
 		match ev {
 			AsyncGitNotification::Diff => self.update_diff()?,
 			AsyncGitNotification::Status => self.update_status()?,
